// pgp4pine 1.45
// Copyright (C) 1998 by Chris Wiegand

// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version. This program is distributed in the hope that it will be useful,but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details. You should have received a copy of the GNU General Public Licensealong
// with this program; if not, write to the Free Software Foundation, Inc., 59
// Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "includes.h"

// -----------------------------------------------------------------

void extractEmailAddress(char *tmpString)
{
	char newStr[EMAIL_ADDRESS_MAX_LENGTH];
	if (strchr(tmpString,'<') != NULL && strchr(tmpString,'>') != NULL) {
		// okay, we've got a realName <email> line...
		strcpy(newStr,strtok(tmpString,"<"));
		strcpy(newStr,strtok('\0',">"));
		strcpy(tmpString,newStr);
	}
}

void clearScreen()
{
#ifndef NCURSES_OKAY
	int i;
	for (i=1;i<60;i++) printf("\n");
#else
	system("clear");
#endif
}

int runProcessFG(const char commandToRun[MAX_COMMAND_LINE_LENGTH])
{
	int c;
	sprintf(debugLine,"runProcessFG: Executing: -%s-\n",commandToRun); 
	documentStatus(debugLine);
	c = system(commandToRun);
	sprintf(debugLine,"runProcessFG: Result Code: %i\n",c); 
	documentStatus(debugLine);
	return c;
}

int askInteger(const char prompt[1024],int defaultVal)
{
	char inChar[3];
	int returnVal;
	int c;
	char ch;

	// This function is meant to ask a Y/N question...
	// I think I'm amazed... CW
	// I don't really understand it, seems to read from itself twice, but
	// that's okay, it works, and it works great. I'll study it someday. :)
	// This code is from Sean Farley.
	do {
		printf("%s",prompt);
		if (defaultVal > 0) { printf(" [Y/n] ? "); } else { printf(" [y/N] ? "); }
		fgets(inChar, 3, stdin);

		// Eat up loose characters in stdin buffer
		if ((inChar[0] != '\n') && (inChar[1] != '\n'))
		{
			while ((c = fgetc(stdin)) != EOF)
			{
				if ((char) c == '\n')
				{
					break;
				}
			}

			if (c == EOF)
			{
				fprintf(stderr, "fgetc() failed\n");
			}
		}
	} while (inChar[0] != '\n' && tolower(inChar[0]) != 'y' && tolower(inChar[0]) != 'n');
	ch = tolower(inChar[0]);
	returnVal = defaultVal;
	// ch is a char, remember?
	if (strncmp(&ch,"y",1) == 0) returnVal = 1;
	if (strncmp(&ch,"n",1) == 0) returnVal = -1;
	return returnVal;
}

char askAlphaRange(const char prompt[1024],char defaultVal,char maxVal)
{
	char inChar[3];
	int c, canLeave=0;

	// This function is meant to ask a a-z/A-Z question...
	// I think I'm amazed... CW - it's modified from askInteger (above)
	// I don't really understand it, seems to read from itself twice, but
	// that's okay, it works, and it works great. I'll study it someday. :)
	// This code is from Sean Farley modified by Chris Wiegand.
	do {
		printf("%s",prompt);
		printf(" [%c] ? ",tolower(defaultVal));
		fgets(inChar, 3, stdin);

		// Eat up loose characters in stdin buffer
		if ((inChar[0] != '\n') && (inChar[1] != '\n'))
		{
			while ((c = fgetc(stdin)) != EOF)
			{
				if ((char) c == '\n')
				{
					break;
				}
			}

			if (c == EOF)
			{
				fprintf(stderr, "fgetc() failed\n");
			}
		}
		if (isalpha(inChar[0]) != 0) {
			// a-zA-Z
			if (tolower(maxVal) >= tolower(inChar[0])) canLeave = (int) inChar[0];
		}
		if (inChar[0] == '\n') canLeave = 1;
	} while (canLeave == 0);
	if (canLeave == 1) canLeave = defaultVal;
	return (char) canLeave;
}

void waitForReturn()
{
	char tmpChar;
	printf("\nPlease press Enter / Return a couple times to continue.\n");
	printf("If you want to cancel, press Control-C.\n");
	tmpChar = getchar(); // first, to clear the cache...
	tmpChar = getchar(); // then for real.
	printf("\n");
}

// ---------------------------------------------------------------------------------

void printUsage()
{
	printf("Usage: pgp4pine (options)\n");
	printf("-e: Encrypt / Sign file  <-=OR=->  -d: Decrypt / Verify file\n");
	printf("-i <file>: Incoming file\n");
	printf("-o <file>: Outgoing file (OPTIONAL)\n");
	printf("-t <file>: Temporary file (OPTIONAL)\n");
	printf("-f: Use pipes instead of incoming and outgoing files. *UNTESTED*\n");
	printf("-k: Keep temp files (or incoming/outgoing files) after done. *UNTESTED*\n");
	printf("-r <recipients>: Recipients (OPTIONAL)\n");
	printf("Note: recipients MUST be last!\n");
	printf("Type \"man pgp4pine\" for more options.\n");
}

void printVersion()
{
#ifndef MY_DEBUG
	printf("pgp4pine 1.44\n");
#else
	printf("pgp4pine 1.44 (DEBUG)\n");
#endif
	printf("Written by Chris Wiegand <cwiegand@urgentmail.com>\n");
	printf("Current homepage: http://www.dimensional.com/~cwiegand/linux/pgp4pine.html\n");
	printf("This program is protected by the GPL v2.\n");
}

void pipeSTDINToFile(const char *fileName)
{
	FILE *fp;
	char ch;
	if ((fp = fopen(fileName,"w")) == NULL) {
		fprintf(stderr,"Sorry, but I was unable to open the temp pipe file for input.\n");
	} else {
		do {
			ch = fgetc(stdin);
			if (ch != EOF) fputc(ch,fp);
		} while (ch != EOF);
		fclose(fp);
		fclose(stdin); // I'm assuming this is necessary...
	}
}

void *myMalloc(size_t size)
{
	void *newPtr;
	newPtr = malloc(size);
	if (!newPtr) {
		fprintf(stderr,"Fatal Error: not able to malloc (%d bytes)!\n",size);
		exit(-2);
	}
	return newPtr;
}

void myFree(void *oldPtr)
{
	if (!oldPtr) free(oldPtr);
}	

void stillAlive()
{
	printf(".");
	fflush(stdout);
}

void killFirstChar(char *inString)
{
	int c;
	// inString is passed back, so it's a pointer
	for (c=0;c<strlen(inString);c++) inString[c] = inString[c+1];
}

void cpString(FILE *inFile, char buf[65530])
{
	char ch;
	int i = 0;
	
	ch = fgetc(inFile);
	
	while((ch != EOF) && (ch != '\n'))
	{
		buf[i] = ch;
		ch = fgetc(inFile);
		i++;
	}
	buf[i] = '\0';
}
