// pgp4pine keyrings.c

// See common.c for the version and license.
// Copyright (C) 1998 by Chris Wiegand

#include "includes.h"

// -----------------------------------------------------------------

void deallocAllKeys(struct pkiKey *killKey)
{
	struct pkiKey *thisKey = NULL;
	struct pkiKey *nextKey2 = NULL;
	thisKey = killKey; // copy the pointer...
	sprintf(debugLine,"%p, ",thisKey);
	if (thisKey != NULL) {
		do {
			sprintf(debugLine,"%p, ",thisKey);
			documentStatus(debugLine);
			nextKey2 = thisKey->nextKey;
			myFree(thisKey);
			thisKey = nextKey2;
		} while (thisKey != NULL);
	}
	documentStatus("Done with this keyring.\n");
}

void getSecretKeyring()
{
	struct pkiKey *myNewKey = NULL;
	struct pkiKey *firstKey = NULL;
	struct pkiKey *oldKey = NULL;
	FILE *inFile, *outFile;
	char buf[CONSOLE_IO_LINE_LENGTH], keyType[CONSOLE_IO_LINE_LENGTH];
	char tmpString[CONSOLE_IO_LINE_LENGTH];
	int lineType, inSec = 0;
	
	printf("Getting secret key(s) information");

	switch (myUserPrefs->pgpVersion) {
	case 5:
		pipe_pgp("pgpk -l",&inFile,&outFile);
		while (!feof(outFile)) {
			// fscanf(outFile,"%s",buf);
			// using fgets() causes problems...
			fgets(buf,sizeof(buf),outFile);
			strcpy(keyType,strtok(buf," "));
			sprintf(debugLine,"getSecretKey: lineIn: %s",buf);
			documentStatus(debugLine);
			lineType = 0; // default, unknown line type
			if (strcmp(keyType,"sec+") == 0 || strcmp(keyType,"sec") == 0) lineType = 1; // secret line...
			if (strcmp(keyType,"pub") == 0) lineType = 2; // public key
			if (strcmp(keyType,"sub") == 0) lineType = 3; // DH/DSS sub-key
			if (strcmp(keyType,"uid") == 0) lineType = 4; // user id
			if (strcmp(keyType,"sig") == 0) lineType = 5; // signature 
			switch (lineType) {
			case 3:
				break;
			case 5:
				break;
			case 1:
				inSec = 1;
				stillAlive();
				myNewKey = (struct pkiKey *) myMalloc(sizeof(pkiKeyStruct));
				if (firstKey == NULL) firstKey = myNewKey;
				if (oldKey != NULL) oldKey->nextKey = myNewKey;
				oldKey = myNewKey;

				// next, key size...
				strcpy(tmpString,strtok('\0'," "));
				strcpy(myNewKey->keySize,tmpString);

				// next, keyID
				strcpy(tmpString,strtok('\0'," "));
				strcpy(myNewKey->keyID,tmpString);
				
				// next, 2 dates
				strcpy(tmpString,strtok('\0'," "));
				strcpy(tmpString,strtok('\0'," "));
				// I don't use them...
				
				// next, key type...
				strcpy(tmpString,strtok('\0'," "));
				strcpy(myNewKey->keyType,tmpString);
				break;
			case 4:
				if (inSec == 1) {
					strcpy(tmpString,strtok('\0',"\n"));
					killFirstChar(tmpString);
					strcpy(myNewKey->displayName,tmpString);
					extractEmailAddress(tmpString);
					strcpy(myNewKey->emailAddress,tmpString);
				}
				break;
			case 2:
			case 0:
				// this is a public key, or other junk we don't care about, however, it may be subs or uids under
				// it, so we set myNewKey to junkKey, which we dump later...
				inSec = 0;
				break;
			}
		}
		fclose(outFile);
		break;
	case 2:
		pipe_pgp("pgp -kv ~/.pgp/secring.pgp",&inFile,&outFile);
		while (!feof(outFile)) {
			// fscanf(outFile,"%s",buf);
			// using fgets() causes problems...
			fgets(buf,sizeof(buf),outFile);
			strcpy(keyType,strtok(buf," "));
			sprintf(debugLine,"getSecretKey: lineIn: %s",buf);
			documentStatus(debugLine);
			if (strcmp(keyType,"sec") == 0) {
				stillAlive();
				myNewKey = (struct pkiKey *) myMalloc(sizeof(pkiKeyStruct));
				if (firstKey == NULL) firstKey = myNewKey;
				if (oldKey != NULL) oldKey->nextKey = myNewKey;
				oldKey = myNewKey;

				// next, key size...
				strcpy(tmpString,strtok('\0',"/"));
				strcpy(myNewKey->keySize,tmpString);

				// next, keyID
				strcpy(tmpString,strtok('\0'," "));
				strcpy(myNewKey->keyID,tmpString);
				
				// next, 1 date
				strcpy(tmpString,strtok('\0'," "));
				// I don't use it...
				
				// next, key type...
				strcpy(myNewKey->keyType,"RSA");

				// finally, user name
				strcpy(tmpString,strtok('\0',"\n"));
				strcpy(myNewKey->displayName,tmpString);
				extractEmailAddress(tmpString);
				strcpy(myNewKey->emailAddress,tmpString);
			}
		}
		fclose(outFile);
		break;
	case 1:
		pipe_pgp("gpg --list-secret-keys --with-colons --no-greeting",&inFile,&outFile);
		while (!feof(outFile)) {
			// fscanf(outFile,"%s",buf);
			// using fgets() causes problems...
			fgets(buf,sizeof(buf),outFile);
			if (strchr(buf,':') != NULL) {
				strcpy(keyType,strtok(buf,":"));
				sprintf(debugLine,"getSecretKey: lineIn: %s",buf);
				documentStatus(debugLine);
				if (strcmp(keyType,"sec") == 0) {
					stillAlive();
					myNewKey = (struct pkiKey *) myMalloc(sizeof(pkiKeyStruct));
					if (firstKey == NULL) firstKey = myNewKey;
					if (oldKey != NULL) oldKey->nextKey = myNewKey;
					oldKey = myNewKey;
	
					// in --list-keys, this is filled in, but not with --list-secret-keys. <shrug>
					// strcpy(tmpString,strtok('\0',":"));
	
					// next, key size...
					strcpy(tmpString,strtok('\0',":"));
					strcpy(myNewKey->keySize,tmpString);

					// next, GPG key type...
					strcpy(tmpString,strtok('\0',":"));
					strcpy(myNewKey->keyType,"Unknown");
					if (strcmp(tmpString,"1") == 0) strcpy(myNewKey->keyType,"RSA");
					if (strcmp(tmpString,"16") == 0) strcpy(myNewKey->keyType,"ElGamal");
					if (strcmp(tmpString,"17") == 0) strcpy(myNewKey->keyType,"DSA");
					if (strcmp(tmpString,"20") == 0) strcpy(myNewKey->keyType,"ElGamal");
			
					// keyID
					strcpy(tmpString,strtok('\0',":"));
					strcpy(myNewKey->keyID,tmpString);
					
					// next, 1 date
					strcpy(tmpString,strtok('\0',":"));
					// I don't use it...

					// an empty field, strtok doesn't see it because it's always blank...
					// strcpy(tmpString,strtok('\0',":"));
					
					// finally, user name
					strcpy(tmpString,strtok('\0',":")); // last colon...
					strcpy(myNewKey->displayName,tmpString);
					extractEmailAddress(tmpString);
					strcpy(myNewKey->emailAddress,tmpString);
				}
			}
		}
		fclose(outFile);
		break;
	}
	printf("\n");
	secretKeyring = firstKey;
}

void chooseSecretKey()
{
	char retValue[EMAIL_ADDRESS_MAX_LENGTH];
	strcpy(retValue,"");

	// now, let's ask which one they want to use...
	if (secretKeyring->nextKey != NULL) {
		// multiple keys...
		int i=0, sel = 0, keyCount=1;
		struct pkiKey *currentKey = NULL;

		currentKey = secretKeyring;
		do {
			currentKey = currentKey->nextKey;
			keyCount++;
		} while (currentKey != NULL);
		currentKey = secretKeyring;
		
		// multiple secret keys...
		printf("\nYou have multiple secret keys. Listed below are the keys. Please select one.\n");
		do {
			i++;
			printKeyInfo((char) (i+'a'-1),currentKey);
			currentKey = currentKey->nextKey;
		} while (currentKey != NULL);
		printf("\n");
		sel = (askAlphaRange("Please enter a valid selection: ",'a',(i+'a'-1))-'a'+1);
		currentKey = secretKeyring; // set back to beginning...
		for (i=1;i<sel;i++) currentKey = currentKey->nextKey; // skip to key we want

#ifdef USE_DISPLAY_NAME
		strcpy(retValue,currentKey->displayName);
#else
		strcpy(retValue,currentKey->keyID);
#endif

		strcpy(mySecretKey,retValue);
		documentStatus("Secret key chosen! Ah, the harmony in life!!!!!\n");
		printf(" ... Please wait ...\n");
	} else {
		if (secretKeyring == NULL) {
			// no secret keys found!!!
			printf("\nWarning! It appears that you don't have a secret key defined! Please see\n your PGP/GPG documentation for assistance in making a key pair!\n");
		} else {
			// only one secret key...
#ifdef USE_DISPLAY_NAME
			strcpy(retValue,secretKeyring->displayName);
#else
			strcpy(retValue,secretKeyring->keyID);
#endif
			strcpy(mySecretKey,retValue);
		}
	}
	documentStatus("Leaving secret area!\n");
}

void getPublicKeyring()
{
	struct pkiKey *myNewKey = NULL;
	struct pkiKey *firstKey = NULL;
	struct pkiKey *oldKey = NULL;
	struct pkiKey *goodKey = NULL;
	FILE *inFile, *outFile;
	char buf[65530], keyType[CONSOLE_IO_LINE_LENGTH];
	char tmpString[CONSOLE_IO_LINE_LENGTH];
	
	printf("Getting public key(s) information");

	switch (myUserPrefs->pgpVersion) {
	case 5:
		goodKey = (struct pkiKey *) myMalloc(sizeof(pkiKeyStruct));
		pipe_pgp("pgpk -l",&inFile,&outFile);
		while (!feof(outFile)) {
			// fscanf(outFile,"%s",buf);
			// using fgets() causes problems...
			fgets(buf,sizeof(buf),outFile);
			strcpy(keyType,strtok(buf," "));
			sprintf(debugLine,"getSecretKey: lineIn: %s",buf);
			documentStatus(debugLine);
			if (strcmp(keyType,"sec") == 0 || strcmp(keyType,"sec+") == 0 || strcmp(keyType,"pub") == 0) {
				// next, key size...
				strcpy(tmpString,strtok('\0'," "));
				strcpy(goodKey->keySize,tmpString);

				// next, keyID
				strcpy(tmpString,strtok('\0'," "));
				strcpy(goodKey->keyID,tmpString);
				
				// next, 2 dates
				strcpy(tmpString,strtok('\0'," "));
				strcpy(tmpString,strtok('\0'," "));
				// I don't use them...
				
				// next, key type...
				strcpy(tmpString,strtok('\0'," "));
				strcpy(goodKey->keyType,tmpString);
			}
			if (strcmp(keyType,"uid") == 0) {
				stillAlive();
				myNewKey = (struct pkiKey *) myMalloc(sizeof(pkiKeyStruct));
				if (firstKey == NULL) firstKey = myNewKey;
				if (oldKey != NULL) oldKey->nextKey = myNewKey;
				myNewKey->nextKey = NULL;
				oldKey = myNewKey;
				
				strcpy(tmpString,strtok('\0',"\n"));
				killFirstChar(tmpString);
				strcpy(myNewKey->displayName,tmpString);
				extractEmailAddress(tmpString);
				strcpy(myNewKey->emailAddress,tmpString);
				
				// filler
				strcpy(myNewKey->keyID,goodKey->keyID);
				strcpy(myNewKey->keySize,goodKey->keySize);
				strcpy(myNewKey->keyType,goodKey->keyType);
			}
		}
		myFree(goodKey);
		fclose(outFile);
		break;
	case 2:
		pipe_pgp("pgp -kv",&inFile,&outFile);
		while (!feof(outFile)) {
			// fscanf(outFile,"%s",buf);
			// using fgets() causes problems...
			fgets(buf,sizeof(buf),outFile);
			strcpy(keyType,strtok(buf," "));
			sprintf(debugLine,"getSecretKey: lineIn: %s",buf);
			documentStatus(debugLine);
			if (strcmp(keyType,"sec") == 0 || strcmp(keyType,"pub") == 0) {
				stillAlive();
				myNewKey = (struct pkiKey *) myMalloc(sizeof(pkiKeyStruct));
				if (firstKey == NULL) firstKey = myNewKey;
				if (oldKey != NULL) oldKey->nextKey = myNewKey;
				myNewKey->nextKey = NULL;
				oldKey = myNewKey;

				// next, key size...
				strcpy(tmpString,strtok('\0',"/"));
				strcpy(myNewKey->keySize,tmpString);

				// next, keyID
				strcpy(tmpString,strtok('\0'," "));
				strcpy(myNewKey->keyID,tmpString);
				
				// next, 1 date
				strcpy(tmpString,strtok('\0'," "));
				// I don't use it...
				
				// next, key type...
				strcpy(myNewKey->keyType,"RSA");

				// finally, user name
				strcpy(tmpString,strtok('\0',"\n"));
				strcpy(myNewKey->displayName,tmpString);
				extractEmailAddress(tmpString);
				strcpy(myNewKey->emailAddress,tmpString);
			}
		}
		fclose(outFile);
		break;
	case 1:
		pipe_pgp("gpg --list-keys --with-colons --no-greeting",&inFile,&outFile);
		while (!feof(outFile)) {
			int lineType=0;
			cpString(outFile,buf);
			// using fgets() causes problems...
			// fgets(buf,sizeof(buf),outFile);
			if (strchr(buf,':') != NULL) {
				strcpy(keyType,strtok(buf,":"));
				sprintf(debugLine,"getPublicKey: lineIn: %s",buf);
				documentStatus(debugLine);
				lineType = 0;
				if (strcmp(keyType,"sec+") == 0 || strcmp(keyType,"sec") == 0) lineType = 1; // secret line...
				if (strcmp(keyType,"pub") == 0) lineType = 2; // public key
				if (strcmp(keyType,"uid") == 0) lineType = 4; // user id
				if (lineType == 1 || lineType == 2) {
					stillAlive();
					myNewKey = (struct pkiKey *) myMalloc(sizeof(pkiKeyStruct));
					if (firstKey == NULL) firstKey = myNewKey;
					if (oldKey != NULL) oldKey->nextKey = myNewKey;
					myNewKey->nextKey = NULL;
					oldKey = myNewKey;
					sprintf(debugLine,"new key: %p\n",myNewKey);
					documentStatus(debugLine);

					// I don't know what the next field is, need to check Werner's docs...
					strcpy(tmpString,strtok('\0',":"));
	
					// next, key size...
					strcpy(tmpString,strtok('\0',":"));
					strcpy(myNewKey->keySize,tmpString);

					// next, GPG key type...
					strcpy(tmpString,strtok('\0',":"));
					strcpy(myNewKey->keyType,"Unknown");
					if (strcmp(tmpString,"1") == 0) strcpy(myNewKey->keyType,"RSA");
					if (strcmp(tmpString,"16") == 0) strcpy(myNewKey->keyType,"ElGamal");
					if (strcmp(tmpString,"17") == 0) strcpy(myNewKey->keyType,"DSA");
					if (strcmp(tmpString,"20") == 0) strcpy(myNewKey->keyType,"ElGamal");
			
					// keyID
					strcpy(tmpString,strtok('\0',":"));
					strcpy(myNewKey->keyID,tmpString);
					
					// next, 1 date
					strcpy(tmpString,strtok('\0',":"));
					// I don't use it...
					
					// next, two unused fields...
					strcpy(tmpString,strtok('\0',":"));
					strcpy(tmpString,strtok('\0',":"));
					
					// finally, user name
					strcpy(tmpString,strtok('\0',":")); // last colon...
					strcpy(myNewKey->displayName,tmpString);
					extractEmailAddress(tmpString);
					strcpy(myNewKey->emailAddress,tmpString);
					goodKey = myNewKey;
					break;
				}
				if (lineType == 4) {
					stillAlive();
					myNewKey = (struct pkiKey *) myMalloc(sizeof(pkiKeyStruct));
					if (firstKey == NULL) firstKey = myNewKey;
					if (oldKey != NULL) oldKey->nextKey = myNewKey;
					myNewKey->nextKey = NULL;
					oldKey = myNewKey;
					sprintf(debugLine,"new key: %p\n",myNewKey);
					documentStatus(debugLine);
	
					// finally, user name
					strcpy(tmpString,strtok('\0',":")); // last colon...
					strcpy(myNewKey->displayName,tmpString);
					extractEmailAddress(tmpString);
					strcpy(myNewKey->emailAddress,tmpString);
					// filler
					strcpy(myNewKey->keyID,goodKey->keyID);
					strcpy(myNewKey->keySize,goodKey->keySize);
					strcpy(myNewKey->keyType,goodKey->keyType);
					break;
				}
			}
		}
		fclose(outFile);
		break;
	}
	publicKeyring = firstKey;
}

