// pgp4pine menus.c

// See common.c for the version and license.
// Copyright (C) 1998 by Chris Wiegand

#include "includes.h"

// ---------------------------------------------------------------------------------

void fileVerifyDecryptMenu(const char *inFile,const char *outFile)
{
	// inFile and outFile are a) just passed to fileVerifyDecrypt, and
	// b) should already be copied into inFile, outFile in main.c:main()
	// so they should be (safe) pointers...
	printf("\nDecrypt/Verify file\n\n");
	fileVerifyDecrypt(inFile,outFile);
	waitForReturn();
	return;
}

void fileSignEncryptMenu(const char *inFile, char *outFile, char *recipients)
{
	// all of the pointer arguments above should be "safe" pointers, 
	// as they are given variables in main.c:main()
	
	int signFile,encryptFile,encryptToSelf=0,useASCIIArmor,useUniversalText,useDetachedSigs;
	char tmpList[EMAIL_ADDRESS_MAX_LENGTH];
	
	printf("\nEncrypt/Sign file\n\n");

	// Note: these can be set in ~/.pgp4pinerc, and in the command line (-sf+,-ef-,-af+, etc...)
	signFile = myUserPrefs->autoSign;
	if (signFile == 0) signFile = askInteger("Do you want to sign this file/message? ",1);

	encryptFile = myUserPrefs->autoEncrypt;
	if (encryptFile == 0) encryptFile = askInteger("Do you want to encrypt this file/message? ",-1);

	useASCIIArmor = myUserPrefs->asciiArmor;
	if (useASCIIArmor == 0) useASCIIArmor = askInteger("Do you want to use ASCII (RADIX-64) armoring? ", 1);

	useUniversalText = myUserPrefs->universalText;	
	if (useUniversalText == 0) useUniversalText = askInteger("Do you want to use canonical (universal) text formatting? ",1);

	if (encryptFile > 0) myUserPrefs->detachedSig = -1; // can't detach signature when encrypting...
	useDetachedSigs = myUserPrefs->detachedSig;
	if (useDetachedSigs == 0) useDetachedSigs = askInteger("Do you want to use a detached signature?",-1);

	if (encryptFile > 0) {
		encryptToSelf = myUserPrefs->encryptToSelf;
		if (encryptToSelf == 0) encryptToSelf = askInteger("Do you want to encrypt to yourself, too? ",1);
		if (strlen(recipients) < 3 || prefsOverrideRecipients == 1) {
			strcpy(recipients,"");
			printf("Please enter the recipients below. Type . by itself on it's own line to end.\n");
			do {
				scanf("%s",tmpList);
				// Using fgets seems to break the strcmp() below...
//				fgets(tmpList, EMAIL_ADDRESS_MAX_LENGTH, stdin);
				strcat(recipients,tmpList);
				strcat(recipients," ");
			} while (strcmp(tmpList,"."));
		}
	}

	sprintf(debugLine,"fileSignEncrypt: recipients -%s-\n",recipients);
	documentStatus(debugLine);

	if (useDetachedSigs > 0) {
		// if we ARE just signing and we're detaching it...
		// then we want to force the signature to <inFileName>.sig like pgp2...
		strcpy(outFile,inFile);
		strcat(outFile,".sig");
		myUserPrefs->detachedSig = 1; // I use this in main.c > main() to determine if I detached or not...
	}

	if (strlen(recipients) < 3) encryptFile = -1;
	if (signFile > 0 || encryptFile > 0) 
	{
		fileSignEncrypt(inFile,outFile,signFile,encryptFile,recipients,useASCIIArmor,useUniversalText,encryptToSelf,useDetachedSigs);
		waitForReturn();
	}
	
	if (useDetachedSigs > 0) {
		// we did a detached signature, however, the procedure above us (main usually) will know
		// that we changed outFile to inFile.sig, so we're going to reset outFile again...
		strcpy(outFile,inFile);
		// Since we did a detached signature, we're going to send exactly what we got...
	}
}

